{-# LANGUAGE OverloadedStrings #-}
module Types.Dependency
  ( Action(..)
  , UToken(..)
  , Dependency(..)
  , makeDependency
  , formatDependencyTrees
  ) where

import Types.Token (Token(..))
import Types.Value (Value)
import Types.Function (Function)
import Types.FunctionIO (FunctionIO)

import Text.Printf (printf)
import Data.Text (Text)
import qualified Data.Text as T
import Language.Haskell.TH.Syntax (Lift)

data Action = Function Function
            | FunctionIO FunctionIO
            | Inject Value
            | FilterComp
            | GetListElem
            | SetListElem
            | UnzipFst
            | UnzipSnd
            | MapComp [Dependency]
  deriving (Show, Lift)

data UToken = UToken Int
            | UTupleToken (UToken, UToken)
            | UZipToken (UToken, UToken)
            | UNoToken
  deriving (Show, Lift)

data Dependency = Dependency UToken Action UToken
  deriving (Show, Lift)

makeDependency :: Token a -> Action -> Token b -> Dependency
makeDependency a action b = Dependency (makeUToken a) action (makeUToken b)

makeUToken :: Token a -> UToken
makeUToken = \case
  Token i -> UToken i
  TupleToken (a, b) -> UTupleToken (makeUToken a, makeUToken b)
  ZipToken (a, b) -> UZipToken (makeUToken a, makeUToken b)
  NoToken -> UNoToken

formatDependencyTrees :: [Dependency] -> Text
formatDependencyTrees = T.concat . (formatDependencyTrees' "")
  where formatDependencyTrees' indentation = concatMap (formatDependencyTree indentation)

        formatDependencyTree indentation (Dependency a action b) =
          concat [ [ indentation ]
                 , formatUToken a
                 , [ " -> " ]
                 , formatUToken b
                 , [ ": " ]
                 , formatAction indentation action
                 ]

        formatUToken = \case
          UToken i ->
            [ T.pack (printf "%02d" i) ]
          UTupleToken (a, b) ->
            concat [ [ "tup(" ]
                   , formatUToken a
                   , [ ", " ]
                   , formatUToken b
                   , [ ")" ]
                   ]
          UZipToken (a, b) ->
            concat [ [ "zip(" ]
                   , formatUToken a
                   , [ ", " ]
                   , formatUToken b
                   , [ ")" ]
                   ]
          UNoToken ->
            [ "--" ]

        formatAction indentation = \case
          MapComp subDeps ->
            concat [ [ "MapComp:\n" ]
                   , formatDependencyTrees' (T.append indentation "|  ") subDeps
                   ]
          action ->
            [ T.pack (show action)
            , "\n"
            ]
